#!/bin/ash
# Simple GUI for Bristol
# Copyright (C) James Budiono 2012
#
# License: GNU GPL Version 3 or later
#
# Note: the bits in Global Inits, which is part of GTK-server bash init code
#       is (C) Peter van Eerten. Everything else is (C) James Budiono
#

### configuration
APPTITLE="Bristol GUI"
EMULATORS="mini explorer voyager memory sonic6 mg1 hammond b3 prophet pro52 pro10 pro1 rhodes rhodesbass \
roadrunner bitone bit99 bit100 stratus trilogy obx obxa axxe odyssey arp2600 solina polysix \
poly800 monopoly vox voxM2 juno jupiter bme700 bm dx sidney"
AUDIO_DRIVERS="jack alsa oss"
MIDI_DRIVERS="jack alsa rawalsa"
DEFAULT_SAMPLING_RATE=48000
DEFAULT_PRIORITY=80

### runtime variables with default values

#---------------------------------------------------------- Global Inits
### global variables for proper GTK-server
GTK= NULL="NULL"
PIPE=/tmp/gtk.bash.$$
#trap 'gtk gtk_server_exit; exit' ERR # stop GTK-server in case of an error

### start GTK-server
gtk-server -fifo=$PIPE &
while [ ! -p $PIPE ]; do continue; done

### GTK-server helper functions
# Assignment function
define() { $2 $3 $4 $5 $6 $7 $8 $9; eval $1="$GTK"; }

# Communicate with GTK-server
gtk()
{
/bin/echo "$@" > $PIPE
read GTK < $PIPE
}

#---------------------------------------------------------- GUI realization

gtk_add_missing() {
	# add GTK+ API not defined in gtk-server.cfg
	gtk gtk_server_define gtk_builder_new NONE WIDGET 0
	gtk gtk_server_define gtk_builder_add_from_file NONE INT 3 WIDGET STRING NULL
	gtk gtk_server_define gtk_builder_add_from_string NONE INT 4 WIDGET STRING INT NULL
	gtk gtk_server_define gtk_builder_get_object NONE WIDGET 2 WIDGET STRING
	gtk gtk_server_define gtk_builder_connect_signals NONE NONE 2 WIDGET NULL		
}

### signal handler - start bristol
start_bristol() {
	sampling_rate= priority= emulator= audio_driver= midi_driver=
	
	define sampling_rate gtk gtk_entry_get_text $SAMPLING_RATE
	define priority gtk gtk_entry_get_text $PRIORITY
	
	# emulators
	for e in $EMULATORS; do
		eval gtk gtk_toggle_button_get_active \$EMU_$e
		if [ $GTK -eq 1 ]; then
			emulator=$e
			break
		fi
	done

	# audio drivers
	for e in $AUDIO_DRIVERS; do
		eval gtk gtk_toggle_button_get_active \$AUDIO_$e
		if [ $GTK -eq 1 ]; then
			audio_driver=$e
			break
		fi
	done

	# audio drivers
	for e in $MIDI_DRIVERS; do
		eval gtk gtk_toggle_button_get_active \$MIDI_$e
		if [ $GTK -eq 1 ]; then
			midi_driver=$e
			break
		fi
	done
	
	echo $sampling_rate
	echo $priority
	echo $emulator
	echo $audio_driver
	echo $midi_driver
	startBristol -${emulator} -audio $audio_driver -midi $midi_driver -rate $sampling_rate -priority $priority -autoconn &
	return 0
}

################### main ###################
### init gui
gtk_add_missing

gtk gtk_init
define BUILDER gtk gtk_builder_new
define RESULT gtk gtk_builder_add_from_file $BUILDER "${0}.xml"
gtk gtk_builder_connect_signals $BUILDER

define MAIN_WINDOW gtk gtk_builder_get_object $BUILDER main_window
define OK_BUTTON gtk gtk_builder_get_object $BUILDER ok-button
define CANCEL_BUTTON gtk gtk_builder_get_object $BUILDER cancel-button

define SAMPLING_RATE gtk gtk_builder_get_object $BUILDER sampling-rate
define PRIORITY gtk gtk_builder_get_object $BUILDER priority

# emulators
for e in $EMULATORS; do
	gtk gtk_builder_get_object $BUILDER $e
	eval EMU_$e=$GTK
done

# audio drivers
for e in $AUDIO_DRIVERS; do
	gtk gtk_builder_get_object $BUILDER audio_$e
	eval AUDIO_$e=$GTK
done

# midi drivers
for e in $MIDI_DRIVERS; do
	gtk gtk_builder_get_object $BUILDER midi_$e
	eval MIDI_$e=$GTK
done

gtk g_object_unref $BUILDER

### connect signals
gtk gtk_server_connect $MAIN_WINDOW delete-event quit
gtk gtk_server_connect $OK_BUTTON clicked ok
gtk gtk_server_connect $CANCEL_BUTTON clicked quit

### preparation
gtk gtk_entry_set_text $SAMPLING_RATE $DEFAULT_SAMPLING_RATE
gtk gtk_entry_set_text $PRIORITY $DEFAULT_PRIORITY

### gtk main loop
gtk gtk_widget_show $MAIN_WINDOW
while true; do 
	define EVENT gtk gtk_server_callback wait
	case $EVENT in
		quit)	break ;;
		ok) start_bristol && break ;;
	esac
done
gtk gtk_server_exit 


